@@ -4,10 +4,11 @@
 
 #pragma once
 
-#include <png.h>
+#include <libpng16/png.h>
 #include <zlib.h>
 
 #include <OpenImageIO/Imath.h>
+#include <OpenImageIO/color.h>
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/fmath.h>
@@ -39,7 +40,7 @@ http://lists.openimageio.org/pipermail/oiio-dev-openimageio.org/2009-April/00065
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 #define ICC_PROFILE_ATTR "ICCProfile"
-
+#define CICP_ATTR "CICP"
 
 namespace PNG_pvt {
 
@@ -223,31 +224,23 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
     int srgb_intent;
     double gamma = 0.0;
     if (png_get_sRGB(sp, ip, &srgb_intent)) {
-        spec.attribute("oiio:ColorSpace", "sRGB");
+        spec.attribute("oiio:ColorSpace", "srgb_rec709_scene");
     } else if (png_get_gAMA(sp, ip, &gamma) && gamma > 0.0) {
         // Round gamma to the nearest hundredth to prevent stupid
         // precision choices and make it easier for apps to make
         // decisions based on known gamma values. For example, you want
         // 2.2, not 2.19998.
         float g = float(1.0 / gamma);
-        g       = roundf(100.0 * g) / 100.0f;
-        spec.attribute("oiio:Gamma", g);
-        if (g == 1.0f)
-            spec.attribute("oiio:ColorSpace", "linear");
-        else
-            spec.attribute("oiio:ColorSpace", Strutil::sprintf("Gamma%.2g", g));
+        g       = roundf(100.0f * g) / 100.0f;
+        set_colorspace_rec709_gamma(spec, g);
     } else {
         // If there's no info at all, assume sRGB.
-        spec.attribute("oiio:ColorSpace", "sRGB");
+        spec.attribute("oiio:ColorSpace", "srgb_rec709_scene");
     }
 
     if (png_get_valid(sp, ip, PNG_INFO_iCCP)) {
-        png_charp profile_name = NULL;
-#if OIIO_LIBPNG_VERSION > 10500 /* PNG function signatures changed */
-        png_bytep profile_data = NULL;
-#else
-        png_charp profile_data = NULL;
-#endif
+        png_charp profile_name     = nullptr;
+        png_bytep profile_data     = nullptr;
         png_uint_32 profile_length = 0;
         int compression_type;
         png_get_iCCP(sp, ip, &profile_name, &compression_type, &profile_data,
@@ -257,23 +250,23 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
                            TypeDesc(TypeDesc::UINT8, profile_length),
                            profile_data);
             std::string errormsg;
-            bool ok = decode_icc_profile(
-                cspan<uint8_t>((const uint8_t*)profile_data, profile_length),
-                spec, errormsg);
-            if (!ok) {
-                // errorfmt("Could not decode ICC profile: {}\n", errormsg);
-                // return false;
-                // Nah, just skip an ICC specific error?
+            bool ok
+                = decode_icc_profile(make_cspan(profile_data, profile_length),
+                                     spec, errormsg);
+            if (!ok && OIIO::get_int_attribute("imageinput:strict")) {
+                errorfmt("Could not decode ICC profile: {}\n", errormsg);
+                return false;
             }
         }
     }
 
     png_timep mod_time;
     if (png_get_tIME(sp, ip, &mod_time)) {
-        std::string date = Strutil::sprintf("%4d:%02d:%02d %02d:%02d:%02d",
-                                            mod_time->year, mod_time->month,
-                                            mod_time->day, mod_time->hour,
-                                            mod_time->minute, mod_time->second);
+        std::string date
+            = Strutil::fmt::format("{:4d}:{:02d}:{:02d} {:02d}:{:02d}:{:02d}",
+                                   mod_time->year, mod_time->month,
+                                   mod_time->day, mod_time->hour,
+                                   mod_time->minute, mod_time->second);
         spec.attribute("DateTime", date);
     }
 
@@ -303,15 +296,22 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
     int unit;
     png_uint_32 resx, resy;
     if (png_get_pHYs(sp, ip, &resx, &resy, &unit)) {
-        float scale = 1;
         if (unit == PNG_RESOLUTION_METER) {
             // Convert to inches, to match most other formats
-            scale = 2.54 / 100.0;
+            float scale = 2.54f / 100.0f;
+            float rx    = resx * scale;
+            float ry    = resy * scale;
+            // Round to nearest 0.1
+            rx = std::round(10.0f * rx) / 10.0f;
+            ry = std::round(10.0f * ry) / 10.0f;
             spec.attribute("ResolutionUnit", "inch");
-        } else
+            spec.attribute("XResolution", rx);
+            spec.attribute("YResolution", ry);
+        } else {
             spec.attribute("ResolutionUnit", "none");
-        spec.attribute("XResolution", (float)resx * scale);
-        spec.attribute("YResolution", (float)resy * scale);
+            spec.attribute("XResolution", (float)resx);
+            spec.attribute("YResolution", (float)resy);
+        }
     }
 
     float aspect = (float)png_get_pixel_aspect_ratio(sp, ip);
@@ -326,6 +326,20 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
 
     interlace_type = png_get_interlace_type(sp, ip);
 
+#ifdef PNG_cICP_SUPPORTED
+    {
+        png_byte pri = 0, trc = 0, mtx = 0, vfr = 0;
+        if (png_get_cICP(sp, ip, &pri, &trc, &mtx, &vfr)) {
+            const int cicp[4] = { pri, trc, mtx, vfr };
+            spec.attribute(CICP_ATTR, TypeDesc(TypeDesc::INT, 4), cicp);
+            const ColorConfig& colorconfig(ColorConfig::default_colorconfig());
+            string_view interop_id = colorconfig.get_color_interop_id(cicp);
+            if (!interop_id.empty())
+                spec.attribute("oiio:ColorSpace", interop_id);
+        }
+    }
+#endif
+
 #ifdef PNG_eXIf_SUPPORTED
     // Recent version of PNG and libpng (>= 1.6.32, I think) have direct
     // support for Exif chunks. Older versions don't support it, and I'm not
@@ -334,7 +348,7 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
     png_uint_32 num_exif = 0;
     png_bytep exif_data  = nullptr;
     if (png_get_eXIf_1(sp, ip, &num_exif, &exif_data)) {
-        decode_exif(cspan<uint8_t>(exif_data, num_exif), spec);
+        decode_exif(cspan<uint8_t>(exif_data, span_size_t(num_exif)), spec);
     }
 #endif
 
@@ -429,9 +443,9 @@ create_write_struct(png_structp& sp, png_infop& ip, int& color_type,
 {
     // Check for things this format doesn't support
     if (spec.width < 1 || spec.height < 1)
-        return Strutil::sprintf("Image resolution must be at least 1x1, "
-                                "you asked for %d x %d",
-                                spec.width, spec.height);
+        return Strutil::fmt::format("Image resolution must be at least 1x1, "
+                                    "you asked for {} x {}",
+                                    spec.width, spec.height);
     if (spec.depth < 1)
         spec.depth = 1;
     if (spec.depth > 1)
@@ -455,8 +469,8 @@ create_write_struct(png_structp& sp, png_infop& ip, int& color_type,
         spec.alpha_channel = 3;
         break;
     default:
-        return Strutil::sprintf("PNG only supports 1-4 channels, not %d",
-                                spec.nchannels);
+        return Strutil::fmt::format("PNG only supports 1-4 channels, not {}",
+                                    spec.nchannels);
     }
     // N.B. PNG is very rigid about the meaning of the channels, so enforce
     // which channel is alpha, that's the only way PNG can do it.
@@ -554,12 +568,30 @@ put_parameter(png_structp& sp, png_infop& ip, const std::string& _name,
         return true;
     }
 #endif
+
+    // Before handling general named metadata, suppress format-specific
+    // metadata hints meant for other formats that are not meant to be literal
+    // metadata written to the file. This includes anything with a namespace
+    // prefix of "oiio:" or the name of any other file format.
+    auto colonpos = name.find(':');
+    if (colonpos != std::string::npos) {
+        std::string prefix = Strutil::lower(name.substr(0, colonpos));
+        if (prefix != "png" && is_imageio_format_name(prefix))
+            return false;
+        if (prefix == "oiio")
+            return false;
+    }
+
     if (type == TypeDesc::STRING) {
+        // We can save arbitrary string metadata in multiple png text entries.
+        // Is that ok? Should we also do it for other types by converting to
+        // string?
         png_text t;
         t.compression = PNG_TEXT_COMPRESSION_NONE;
         t.key         = (char*)ustring(name).c_str();
         t.text        = *(char**)data;  // Already uniquified
         text.push_back(t);
+        return true;
     }
 
     return false;
@@ -572,7 +604,8 @@ put_parameter(png_structp& sp, png_infop& ip, const std::string& _name,
 ///
 inline const std::string
 write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
-           std::vector<png_text>& text, bool& convert_alpha, float& gamma)
+           std::vector<png_text>& text, bool& convert_alpha, bool& srgb,
+           float& gamma)
 {
     // Force either 16 or 8 bit integers
     if (spec.format == TypeDesc::UINT8 || spec.format == TypeDesc::INT8)
@@ -594,14 +627,34 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
     convert_alpha = spec.alpha_channel != -1
                     && !spec.get_int_attribute("oiio:UnassociatedAlpha", 0);
 
-    gamma = spec.get_float_attribute("oiio:Gamma", 1.0);
+    string_view colorspace = spec.get_string_attribute("oiio:ColorSpace",
+                                                       "srgb_rec709_scene");
+    const ColorConfig& colorconfig(ColorConfig::default_colorconfig());
+    OIIO_MAYBE_UNUSED bool wrote_colorspace = false;
+    srgb                                    = false;
+    if (colorconfig.equivalent(colorspace, "srgb_rec709_scene")) {
+        srgb  = true;
+        gamma = 1.0f;
+    } else if (colorconfig.equivalent(colorspace, "g22_rec709_scene")) {
+        gamma = 2.2f;
+    } else if (colorconfig.equivalent(colorspace, "g24_rec709_scene")) {
+        gamma = 2.4f;
+    } else if (colorconfig.equivalent(colorspace, "g18_rec709_scene")) {
+        gamma = 1.8f;
+    } else {
+        gamma = spec.get_float_attribute("oiio:Gamma", 1.0f);
+        // obsolete "oiio:Gamma" attrib for back compatibility
+    }
 
-    string_view colorspace = spec.get_string_attribute("oiio:ColorSpace");
-    if (Strutil::iequals(colorspace, "Linear")) {
+    if (colorconfig.equivalent(colorspace, "scene_linear")
+        || colorconfig.equivalent(colorspace, "lin_rec709_scene")) {
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA chunk";
         png_set_gAMA(sp, ip, 1.0);
+        srgb             = false;
+        wrote_colorspace = true;
     } else if (Strutil::istarts_with(colorspace, "Gamma")) {
+        // Back compatible, this is DEPRECATED(3.1)
         Strutil::parse_word(colorspace);
         float g = Strutil::from_string<float>(colorspace);
         if (g >= 0.01f && g <= 10.0f /* sanity check */)
@@ -609,30 +662,43 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA chunk";
         png_set_gAMA(sp, ip, 1.0f / gamma);
-    } else if (Strutil::iequals(colorspace, "sRGB")) {
+        srgb             = false;
+        wrote_colorspace = true;
+    } else if (colorconfig.equivalent(colorspace, "g22_rec709_scene")) {
+        gamma = 2.2f;
+        if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
+            return "Could not set PNG gAMA chunk";
+        png_set_gAMA(sp, ip, 1.0f / gamma);
+        srgb             = false;
+        wrote_colorspace = true;
+    } else if (colorconfig.equivalent(colorspace, "g18_rec709_scene")) {
+        gamma = 1.8f;
+        if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
+            return "Could not set PNG gAMA chunk";
+        png_set_gAMA(sp, ip, 1.0f / gamma);
+        srgb             = false;
+        wrote_colorspace = true;
+    } else if (colorconfig.equivalent(colorspace, "srgb_rec709_scene")) {
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA and cHRM chunk";
         png_set_sRGB_gAMA_and_cHRM(sp, ip, PNG_sRGB_INTENT_ABSOLUTE);
+        srgb             = true;
+        wrote_colorspace = true;
     }
 
     // Write ICC profile, if we have anything
     const ParamValue* icc_profile_parameter = spec.find_attribute(
         ICC_PROFILE_ATTR);
-    if (icc_profile_parameter != NULL) {
+    if (icc_profile_parameter != nullptr) {
         unsigned int length = icc_profile_parameter->type().size();
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG iCCP chunk";
-#if OIIO_LIBPNG_VERSION > 10500 /* PNG function signatures changed */
         unsigned char* icc_profile
             = (unsigned char*)icc_profile_parameter->data();
-        if (icc_profile && length)
+        if (icc_profile && length) {
             png_set_iCCP(sp, ip, "Embedded Profile", 0, icc_profile, length);
-#else
-        char* icc_profile = (char*)icc_profile_parameter->data();
-        if (icc_profile && length)
-            png_set_iCCP(sp, ip, (png_charp) "Embedded Profile", 0, icc_profile,
-                         length);
-#endif
+            wrote_colorspace = true;
+        }
     }
 
     if (false && !spec.find_attribute("DateTime")) {
@@ -640,11 +706,11 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
         time(&now);
         struct tm mytm;
         Sysutil::get_local_time(&now, &mytm);
-        std::string date = Strutil::sprintf("%4d:%02d:%02d %02d:%02d:%02d",
-                                            mytm.tm_year + 1900,
-                                            mytm.tm_mon + 1, mytm.tm_mday,
-                                            mytm.tm_hour, mytm.tm_min,
-                                            mytm.tm_sec);
+        std::string date
+            = Strutil::fmt::format("{:4d}:{:02d}:{:02d} {:02d}:{:02d}:{:02d}",
+                                   mytm.tm_year + 1900, mytm.tm_mon + 1,
+                                   mytm.tm_mday, mytm.tm_hour, mytm.tm_min,
+                                   mytm.tm_sec);
         spec.attribute("DateTime", date);
     }
 
@@ -687,6 +753,25 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
                      (png_uint_32)(yres * scale), unittype);
     }
 
+#ifdef PNG_cICP_SUPPORTED
+    // Only automatically determine CICP from oiio::ColorSpace if we didn't
+    // write colorspace metadata yet.
+    const ParamValue* p = spec.find_attribute(CICP_ATTR,
+                                              TypeDesc(TypeDesc::INT, 4));
+    cspan<int> cicp     = (p) ? p->as_cspan<int>()
+                          : (!wrote_colorspace) ? colorconfig.get_cicp(colorspace)
+                                                : cspan<int>();
+    if (!cicp.empty()) {
+        png_byte vals[4];
+        for (int i = 0; i < 4; ++i)
+            vals[i] = static_cast<png_byte>(cicp[i]);
+        if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
+            return "Could not set PNG cICP chunk";
+        // libpng will only write the chunk if the third byte is 0
+        png_set_cICP(sp, ip, vals[0], vals[1], (png_byte)0, vals[3]);
+    }
+#endif
+
 #ifdef PNG_eXIf_SUPPORTED
     std::vector<char> exifBlob;
     encode_exif(spec, exifBlob, endian::big);
